window.Gleam.initSearch([{"doc":"outcome","title":"outcome","content":"# outcome\n\n[![Package Version](https://img.shields.io/hexpm/v/outcome)](https://hex.pm/packages/outcome)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/outcome/)\n\nAn Error Handling library for Gleam. Inspired by\n&lt;https://github.com/lpil/snag&gt; and &lt;https://effect.website/&gt;.\n\nOutcomes gives you:\n\n- An error stack that makes it easier to track the path of an error.\n- A way to distinguish between unexpected errors (Defect) and expected errors (Failure).\n\n```sh\ngleam add outcome\n```\n\n```gleam\nimport outcome.{type Outcome}\n\nfn using(email) {\n  case signup(email) {\n    Error(stack) -&gt; io.print(outcome.pretty_print(stack))\n    Ok(user) -&gt; {\n      //...\n    }\n  }\n}\n\nfn signup(email: String) -&gt; Outcome(User, String) {\n  use valid_email &lt;- result.try(\n    validate_email(email)\n    |&gt; outcome.with_context(&quot;In signup&quot;)\n  )\n\n  create_user(valid_email)\n}\n\n// An expected error should be marked as a failure\nfn validate_email(email: String) -&gt; Outcome(String, String) {\n  Error(&quot;Invalid email&quot;)\n    |&gt; outcome.into_failure\n}\n\n// An unexpected error should be marked as a defect\nfn create_user() -&gt; Outcome(User, String) {\n  Error(&quot;Some SQL error&quot;)\n  |&gt; outcome.into_defect\n  |&gt; outcome.with_context(&quot;In create_user&quot;)\n}\n```\n\n```gleam\nusing(&quot;invalid&quot;)\n\nFailure: Invalid email\n\nstack:\n  Context: In signup\n  Failure: Invalid email\n```\n\n```gleam\nusing(&quot;sam@sample.com&quot;)\n\nDefect: Some SQL error\n\nstack:\n  c: In signup\n  c: In create_user\n  d: Some SQL error\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/outcome&gt;.\n","url":"index.html"},{"doc":"outcome","title":"ContextStack","content":"pub type ContextStack =\n  List(String)\n A list of context entries\n\nSynonyms:\noutcome.ContextStack\noutcome ContextStack","url":"outcome.html#ContextStack"},{"doc":"outcome","title":"Outcome","content":"pub type Outcome(t, err) =\n  Result(t, Problem(err))\n Alias to Result with Problem as error type.\n\nSynonyms:\noutcome.Outcome\noutcome Outcome","url":"outcome.html#Outcome"},{"doc":"outcome","title":"Problem","content":"pub type Problem(err) {\n  Problem(\n    error: err,\n    severity: Severity,\n    original_severity: Severity,\n    stack: ContextStack,\n  )\n}\n The error type ie. `Result(t, Problem)`\n This contains the error, the severity and the context stack.\nProblem(\n  error: err,\n  severity: Severity,\n  original_severity: Severity,\n  stack: ContextStack,\n)\n\n\nSynonyms:\noutcome.Problem\noutcome Problem","url":"outcome.html#Problem"},{"doc":"outcome","title":"Severity","content":"pub type Severity {\n  Defect\n  Failure\n}\n An application error is either a Defect or a Failure.\n A Defect is an unexpected application error, which shouldn&#39;t be shown to the user.\n A Failure is an expected error.\nDefect\n\n\nFailure\n\n\nSynonyms:\noutcome.Severity\noutcome Severity","url":"outcome.html#Severity"},{"doc":"outcome","title":"error_with_defect","content":"pub fn error_with_defect(defect: a) -&gt; Result(b, Problem(a))\n Create a Defect wrapped in an Error\n\n ## Example\n\n \n case something {\n   True -&gt; Ok(&quot;Yay&quot;)\n   False -&gt; error_with_defect(&quot;Something went wrong&quot;)\n }\n \n\nSynonyms:\noutcome.error_with_defect\noutcome error_with_defect","url":"outcome.html#error_with_defect"},{"doc":"outcome","title":"error_with_failure","content":"pub fn error_with_failure(failure: a) -&gt; Result(b, Problem(a))\n Create Failure wrapped in an Error\n\n ## Example\n\n \n case something {\n   True -&gt; Ok(&quot;Yay&quot;)\n   False -&gt; error_with_failure(&quot;Invalid input&quot;)\n }\n \n\nSynonyms:\noutcome.error_with_failure\noutcome error_with_failure","url":"outcome.html#error_with_failure"},{"doc":"outcome","title":"extract_error","content":"pub fn extract_error(\n  outcome: Result(a, Problem(b)),\n) -&gt; Result(a, b)\n Remove the Problem wrapping, returning just your error.\nSynonyms:\noutcome.extract_error\noutcome extract_error","url":"outcome.html#extract_error"},{"doc":"outcome","title":"into_defect","content":"pub fn into_defect(result: Result(a, b)) -&gt; Result(a, Problem(b))\n Convert an `Error(String)` into an `Error(Defect)`\n This is useful when you have a `Result(t, String)` and\n want to convert it into a `Result(t, Problem)`\n\n ## Example\n\n \n Error(&quot;Something went wrong&quot;)\n |&gt; into_defect\n \nSynonyms:\noutcome.into_defect\noutcome into_defect","url":"outcome.html#into_defect"},{"doc":"outcome","title":"into_failure","content":"pub fn into_failure(\n  result: Result(a, b),\n) -&gt; Result(a, Problem(b))\n Convert an `Error(String)` into an `Error(Failure)`\n This is useful when you have a `Result(t, String)` and\n want to convert it into a `Result(t, Problem)`\n\n ## Example\n\n \n Error(&quot;Invalid input&quot;)\n |&gt; into_failure\n \nSynonyms:\noutcome.into_failure\noutcome into_failure","url":"outcome.html#into_failure"},{"doc":"outcome","title":"map_error","content":"pub fn map_error(\n  outcome: Result(a, Problem(b)),\n  mapper: fn(b) -&gt; b,\n) -&gt; Result(a, Problem(b))\n Map the error value inside a Problem\nSynonyms:\noutcome.map_error\noutcome map_error","url":"outcome.html#map_error"},{"doc":"outcome","title":"map_into_defect","content":"pub fn map_into_defect(\n  result: Result(a, b),\n  mapper: fn(b) -&gt; c,\n) -&gt; Result(a, Problem(c))\n Convert an `Error(t)` into a wrapped Defect, by using a mapping function\n Similar to into_defect, but takes a function to map the error value to a string\nSynonyms:\noutcome.map_into_defect\noutcome map_into_defect","url":"outcome.html#map_into_defect"},{"doc":"outcome","title":"map_into_failure","content":"pub fn map_into_failure(\n  result: Result(a, b),\n  mapper: fn(b) -&gt; c,\n) -&gt; Result(a, Problem(c))\n Convert an `Error(t)` into a wrapped Failure, by using a mapping function\n Similar to into_defect, but takes a function to map the error value to a string\nSynonyms:\noutcome.map_into_failure\noutcome map_into_failure","url":"outcome.html#map_into_failure"},{"doc":"outcome","title":"pretty_print","content":"pub fn pretty_print(\n  problem: Problem(a),\n  to_s: fn(a) -&gt; String,\n) -&gt; String\n Pretty print a Problem, including the stack.\n The latest problem appears at the top of the stack.\n\n ## Example\n\n \n Error(&quot;Something went wrong&quot;)\n |&gt; into_defect\n |&gt; with_context(&quot;In find user function&quot;)\n |&gt; with_context(&quot;More context&quot;)\n |&gt; pretty_print(function.identity)\n \n\n \n Defect: Something went wrong\n\n stack:\n  More context\n  In find user function\n \nSynonyms:\noutcome.pretty_print\noutcome pretty_print","url":"outcome.html#pretty_print"},{"doc":"outcome","title":"print_line","content":"pub fn print_line(\n  problem: Problem(a),\n  to_s: fn(a) -&gt; String,\n) -&gt; String\n Print problem in one line\n\n ## Example\n\n \n Error(&quot;Something went wrong&quot;)\n |&gt; into_defect\n |&gt; with_context(&quot;In find user function&quot;)\n |&gt; print_line(function.identity)\n \n\n \n Defect: Something went wrong &lt;&lt; In find user function\n \nSynonyms:\noutcome.print_line\noutcome print_line","url":"outcome.html#print_line"},{"doc":"outcome","title":"replace_with_defect","content":"pub fn replace_with_defect(\n  result: Result(a, b),\n  e: c,\n) -&gt; Result(a, Problem(c))\n Replaces an `Error(t)` with an `Error(Defect)`\n\n ## Example\n\n \n Error(Nil)\n |&gt; replace_with_defect(&quot;Something went wrong&quot;)\n \nSynonyms:\noutcome.replace_with_defect\noutcome replace_with_defect","url":"outcome.html#replace_with_defect"},{"doc":"outcome","title":"replace_with_failure","content":"pub fn replace_with_failure(\n  result: Result(a, b),\n  e: c,\n) -&gt; Result(a, Problem(c))\n Replaces any Error(t) with an Error(Failure)\n\n ## Example\n\n \n Error(Nil)\n |&gt; replace_with_failure(&quot;Invalid input&quot;)\n \nSynonyms:\noutcome.replace_with_failure\noutcome replace_with_failure","url":"outcome.html#replace_with_failure"},{"doc":"outcome","title":"to_defect","content":"pub fn to_defect(\n  outcome: Result(a, Problem(b)),\n) -&gt; Result(a, Problem(b))\n Coherce the error into a Defect\n\n ## Example\n\n \n Error(&quot;Invalid Input&quot;)\n |&gt; into_failure\n |&gt; to_defect\n \n\nSynonyms:\noutcome.to_defect\noutcome to_defect","url":"outcome.html#to_defect"},{"doc":"outcome","title":"to_failure","content":"pub fn to_failure(\n  outcome: Result(a, Problem(b)),\n) -&gt; Result(a, Problem(b))\n Coherce the error into a Failure.\n The original entry in the stack remains unchanged.\n\n ## Example\n\n \n Error(&quot;Invalid Input&quot;)\n |&gt; into_defect\n |&gt; to_failure\n \n\nSynonyms:\noutcome.to_failure\noutcome to_failure","url":"outcome.html#to_failure"},{"doc":"outcome","title":"unwrap_failure","content":"pub fn unwrap_failure(problem: Problem(a), default_value: a) -&gt; a\n Use this to show a failure to a user.\n If the Problem is a failure, it will show that\n otherwise it will show the default message given.\n We don&#39;t want to show defect messages to users.\n\n ## Example\n\n \n case result {\n  Ok(value) -&gt; io.debug(&quot;Success&quot;)\n  Error(problem) -&gt; io.error(unwrap_failure(problem, &quot;Something went wrong&quot;))\n }\n \nSynonyms:\noutcome.unwrap_failure\noutcome unwrap_failure","url":"outcome.html#unwrap_failure"},{"doc":"outcome","title":"with_context","content":"pub fn with_context(\n  outcome outcome: Result(a, Problem(b)),\n  context context: String,\n) -&gt; Result(a, Problem(b))\n Add context to an Outcome\n This will add a Context entry to the stack\n\n ## Example\n\n \n Error(&quot;Something went wrong&quot;)\n |&gt; into_defect\n |&gt; with_context(&quot;In find user function&quot;)\n \n\nSynonyms:\noutcome.with_context\noutcome with_context","url":"outcome.html#with_context"},{"doc":"outcome","title":"outcome","content":"","url":"outcome.html"}]);